import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import { is, fromJS } from 'immutable'
import update from 'immutability-helper'
import { Col } from 'antd'
import Utils from '@/utils/utils.js'
import Card from './card'
import './index.scss'

const Container = ({list, handleList, handleMenu, deleteMenu }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)
    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })
    handleList(_cards)
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }
  
  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const editCard = id => {
    const { card } = findCard(id)
    handleMenu(card)
  }

  const copyCard = id => {
    const { card } = findCard(id)
    let copycard = fromJS(card).toJS()

    copycard.uuid = Utils.getuuid()
    copycard.origin = false
    copycard.copyType = 'search'
    copycard.focus = true

    let _val = fromJS(copycard).toJS()

    try {
      _val.uuid = Utils.getuuid()
      _val = window.btoa(window.encodeURIComponent(JSON.stringify(_val)))
    } catch (e) {
      console.warn('Stringify Failure')
      _val = ''
    }

    if (_val) {
      let oInput = document.createElement('input')
      oInput.value = _val
      document.body.appendChild(oInput)
      oInput.select()
      document.execCommand('Copy')
      document.body.removeChild(oInput)
    }

    const { index: overIndex } = findCard(id)

    const _cards = update(cards, { $splice: [[overIndex + 1, 0, copycard]] })

    handleList(_cards, copycard)
  }

  const delCard = id => {
    const { card } = findCard(id)
    deleteMenu(card)
  }

  const [, drop] = useDrop({
    accept: 'search',
    drop(item) {
      if (item.hasOwnProperty('originalIndex')) {
        return
      }

      let newcard = {}
      newcard.uuid = Utils.getuuid()
      newcard.focus = true

      newcard.label = 'label'
      newcard.initval = ''
      newcard.type = item.subType
      newcard.resourceType = '0'
      newcard.options = []
      newcard.orderType = 'asc'
      
      let _match = 'like'
      if (['select', 'link', 'checkcard'].includes(item.subType)) {
        _match = '='
      } else if (['date', 'datemonth'].includes(item.subType)) {
        _match = '>='
      } else if (['dateweek', 'daterange', 'range'].includes(item.subType)) {
        _match = 'between'
      } else if (item.subType === 'group') {
        _match = 'between'
        newcard.items = ['day', 'week', 'month', 'quarter', 'year', 'customized']
      }
      
      newcard.match = _match

      let targetId = ''

      if (item.dropTargetId) {
        targetId = item.dropTargetId
        delete item.dropTargetId
      } else if (cards.length > 0) {
        targetId = cards[cards.length - 1].uuid
      }

      const { index: overIndex, card } = findCard(`${targetId}`)

      if (card) {
        newcard.ratio = card.ratio || 6
        newcard.labelwidth = card.labelwidth || 33.3
      }

      const _cards = update(cards, { $splice: [[overIndex + 1, 0, newcard]] })

      handleList(_cards, newcard)
    }
  })

  return (
    <div ref={drop} className="ant-row">
      {cards.map(card => (
        <Col key={card.uuid} span={card.ratio || 6}>
          <Card
            id={`${card.uuid}`}
            card={card}
            moveCard={moveCard}
            editCard={editCard}
            copyCard={copyCard}
            delCard={delCard}
            findCard={findCard}
          />
        </Col>
      ))}
    </div>
  )
}
export default Container
